const logger = require('../logger');
const { t } = require("../../i18n/trans")
const { ZONE_GLOBAL_ENTITY, USE_DEFAULT } = require("../const")

async function IpFilterRefererUserAgentFilter(self, zoneId, domain, domainConfig, report) {
  const ipFilter = domainConfig.IpFilter;
  const referer = domainConfig.Referer;
  const userAgentFilter = domainConfig.UserAgentFilter;
  let ipFilterFlag = true;
  let refererFlag = true;
  let userAgentFilterFlag = true;
  if (!ipFilter || !ipFilter.FilterRules || ipFilter.FilterRules.length === 0 || ipFilter.Switch === 'off') {
    ipFilterFlag = false;
  }
  if (!referer || !referer.RefererRules || referer.RefererRules.length === 0 || referer.Switch === 'off') {
    refererFlag = false;
  }
  if (!userAgentFilter || !userAgentFilter.FilterRules || userAgentFilter.FilterRules.length === 0 || userAgentFilter.Switch === 'off') {
    userAgentFilterFlag = false;
  }
  const ipFilterRules = ipFilter.FilterRules;
  const refererRules = referer.RefererRules;
  const userAgentFilterRules = userAgentFilter.FilterRules;
  if (ipFilterRules.find(rule => {
    return rule.RuleType !== 'all';
  })) {
    report.Details.push({
      config: t('IP黑白名单(IpFilter)'),
      result: t('失败'),
      detail: t(`EO 不支持按目录配置 IP 黑白名单，本配置无法迁移`)
    });
    logger.error(t(`EO 不支持按目录配置 IP 黑白名单，本配置无法迁移`));
    ipFilterFlag = false;
  }
  if (userAgentFilterRules.find(rule => {
    return rule.RuleType !== 'all';
  })) {
    report.Details.push({
      config: t('UA黑白名单(UserAgentFilter)'),
      result: t('失败'),
      detail: t(`EO 不支持按目录配置 UA 黑白名单，本配置无法迁移`)
    });
    logger.error(t('EO 不支持按目录配置 UA 黑白名单，本配置无法迁移'));
    userAgentFilterFlag = false;
  }
  let param = {
    'ZoneId': zoneId,
    'Entity': domain,
    'SecurityConfig': {
      'IpTableConfig': {
        'IpTableRules': [],
        'Switch': 'on'
      }
    }
  };

  if (ipFilterFlag) {
    ipFilterRules.forEach((rule, index) => {
      param.SecurityConfig.IpTableConfig.IpTableRules.push({
        'MatchFrom': 'ip',
        'MatchContent': rule.Filters.join(','),
        'Action': rule.FilterType === 'blacklist' ? 'drop' : 'trans',
        'Status': 'on',
        'Operator': 'match',
        'RuleName': t(`CDN迁移EO-IP黑白名单-{{index}}-{{domain}}`, {index, domain})
      });
    });
  }

  if (refererFlag) {
    refererRules.forEach(rule => {
      param.SecurityConfig.IpTableConfig.IpTableRules.push({
        'MatchFrom': 'referer',
        'MatchContent': rule.Referers.join(','),
        'Action': rule.RefererType === 'blacklist' ? 'drop' : 'trans',
        'Status': 'on',
        'Operator': 'match',
        'RuleName': t(`CDN迁移EO-referer黑白名单-{{domain}}`, {domain})
      });

      if (rule.AllowEmpty) {
        param.SecurityConfig.IpTableConfig.IpTableRules.push({
          'MatchFrom': 'referer',
          'Action': rule.RefererType === 'blacklist' ? 'drop' : 'trans',
          'Status': 'on',
          'Operator': 'is_empty',
          'MatchContent': '',
          'RuleName': t(`CDN迁移EO-referer黑白名单-空referer-{{domain}}`, {domain})
        });
      }
    });
  }

  if (userAgentFilterFlag) {
    userAgentFilter?.FilterRules.forEach((rule, index) => {
      param.SecurityConfig.IpTableConfig.IpTableRules.push({
        'MatchFrom': 'ua',
        'MatchContent': rule.UserAgents.join(','),
        'Action': rule.FilterType === 'blacklist' ? 'drop' : 'trans',
        'Status': 'on',
        'Operator': 'match',
        'RuleName': t(`CDN迁移EO-UA黑白名单-{{index}}-{{domain}}`, {index, domain})
      });
    });
  }

  try {
    // 新站点，新增域名修改安全配置步骤：解绑到空模板
    const bindToEmptyTemplate = {
      Entities: [domain],
      Operate: USE_DEFAULT,
      OverWrite: true,
      TemplateId: ZONE_GLOBAL_ENTITY,
      ZoneId: zoneId,
    }
    await self.API.eoClient['BindSecurityTemplateToEntity'](bindToEmptyTemplate);
    const res = await self.API.eoClient['ModifySecurityPolicy'](param);
    report.Details.push({
      config: t('安全配置创建(IP黑白名单:IpFilter, 防盗链:Referer, UA黑白名单:UserAgentFilter)'),
      result: t('成功'),
      detail: ``
    });
    return res;
  }
  catch (e) {
    report.Details.push({
      config: t('安全配置创建(IP黑白名单:IpFilter, 防盗链:Referer, UA黑白名单:UserAgentFilter)'),
      result: t('失败'),
      detail: `${e.toString()}`
    });
    logger.error(e);
  }
}

module.exports = IpFilterRefererUserAgentFilter;